/*
 * Copyright © 2015 Robert Bosch Car Multimedia GmbH
 *
 * SPDX-License-Identifier: MPL-2.0
 * This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of the MPL was not distributed with this
 * file, You can obtain one at http://mozilla.org/MPL/2.0/.
 */

#include <glib.h>
#include <glib/gstdio.h>
#include <stdio.h>
#include <stdlib.h>
#include <gio/gio.h>
#include "org.apertis.Newport.Service.h"
#include "org.apertis.Newport.Download.h"
#include "newport-enums.h"
/* FIXME: This url has to be changed */
#define URL "http://www.stephaniequinn.com/Music/Allegro%20from%20Duet%20in%20C%20Major.mp3"
#define DOWNLOAD_FILE  "test_file.ogg"
/* FIXME: This url has to be changed */
#define APP_URL "http://alpha.gnu.org/gnu/tar/tar-1.13.25.tar.gz"
#define APP_NAME "Download-Client"




static NewportService *proxy = NULL;

static void
test_newport_complete_download (void);

static gboolean
_default_event_source (gpointer user_data)
{
	  gboolean *timed_out = user_data;
	  *timed_out = TRUE;
	  return G_SOURCE_REMOVE;
}


static void
_download_error_changed (GObject *gobject, GParamSpec *pspec,
                         gpointer user_data)
{
  *(gboolean*) user_data = FALSE;
}

static void
_download_state_changed (GObject *gobject, GParamSpec *pspec,
                         gpointer user_data)
{
guint state=newport_download_get_state (NEWPORT_DOWNLOAD (gobject));
const gchar *url=newport_download_get_url (NEWPORT_DOWNLOAD (gobject));
  if (((g_strcmp0 (url,
  APP_URL) == 0))
      && ((state == NEWPORT_DOWNLOAD_STATE_PAUSED_BY_USER) || (state == NEWPORT_DOWNLOAD_STATE_IN_PROGRESS)))
    {
      *(gboolean *) user_data = TRUE;

    }

  else  if (((g_strcmp0 (url, URL)
      == 0))
      && (state
          == NEWPORT_DOWNLOAD_STATE_SUCCESS))
    {
      *(gboolean *) user_data = TRUE;

    }

}

static void
_download_info (NewportDownload *object, guint64 current_download_size,
                guint64 arg_total_file_size, guint64 arg_download_speed,
                guint64 arg_elapsed_time, guint64 arg_remain_time,
                gpointer user_data)
{

  if (g_strcmp0 (newport_download_get_url (NEWPORT_DOWNLOAD (object)), APP_URL)
      == 0)
    {
      *(gboolean*) user_data = TRUE;
    }

}

static void
test_newport_complete_download (void)
{
  GError *error = NULL;
  gboolean state_result = FALSE;
  gchar *out_obj_path = NULL;
  NewportDownload *download_proxy;
  gboolean result;
  gchar *arg_download_path;
  gboolean timed_out = FALSE;
  guint timeout_id;

  arg_download_path =
    g_build_filename (g_get_user_special_dir (G_USER_DIRECTORY_DOWNLOAD),
                      "newport-client", "musicfile", NULL);
  /* Call  download start to start downloading the file and wait till download-information signal quits the mainloop */
  result = newport_service_call_start_download_sync (proxy, URL,
                                                     arg_download_path,
                                                     &out_obj_path, NULL,
                                                     &error);
  g_free (arg_download_path);
  g_assert_no_error (error);
  g_assert (result);
  download_proxy =
    newport_download_proxy_new_for_bus_sync (G_BUS_TYPE_SESSION,
                                             G_BUS_NAME_WATCHER_FLAGS_NONE,
                                             "org.apertis.Newport",
                                             out_obj_path, NULL, &error);
  g_free (out_obj_path);
  /*state_result flag set in download-information signal */
  g_signal_connect (download_proxy, "notify::error",
                    G_CALLBACK (_download_error_changed), &state_result);
  g_signal_connect (download_proxy, "notify::state",
                    G_CALLBACK (_download_state_changed), &state_result);
  timeout_id = g_timeout_add_seconds (300, _default_event_source, &timed_out);
  while (!timed_out && !state_result)
    g_main_context_iteration (NULL, TRUE);
  g_assert (!timed_out);
  g_source_remove (timeout_id);
  g_assert (state_result);

}

static void
test_newport_get_download_list (void)
{
  gchar **out_paths = NULL;
  newport_service_call_get_downloads_sync (proxy, &out_paths, NULL, NULL);
  g_assert_nonnull (out_paths);
  g_strfreev (out_paths);

}

static void
test_newport_pause_download (void)
{
  GError *error = NULL;
  gboolean state_result = FALSE;
  gchar *out_obj_path = NULL;
  NewportDownload *download_proxy;
  gulong dwn_state_id;
  gboolean result;
  gboolean timed_out = FALSE;
  guint timeout_id;

  newport_service_call_get_download_for_url_sync (proxy, APP_URL, &out_obj_path,
                                                  NULL, &error);
  download_proxy =
    newport_download_proxy_new_for_bus_sync (G_BUS_TYPE_SESSION,
                                             G_BUS_NAME_WATCHER_FLAGS_NONE,
                                             "org.apertis.Newport",
                                             out_obj_path, NULL, NULL);
  g_free (out_obj_path);

  dwn_state_id = g_signal_connect (download_proxy, "notify::state",
                                   G_CALLBACK (_download_state_changed),
                                   &state_result);
  result = newport_download_call_pause_sync (download_proxy, NULL, &error);
  g_assert_no_error (error);
  g_assert (result);
  g_timeout_add_seconds (100, _default_event_source, &timed_out);
  timeout_id = g_timeout_add_seconds (100, _default_event_source, &timed_out);
  while (!timed_out && !state_result)
    g_main_context_iteration (NULL, TRUE);
  g_assert (!timed_out);
  g_source_remove (timeout_id);
  g_assert (state_result);
  g_signal_handler_disconnect (download_proxy, dwn_state_id);
}

static void
test_newport_cancel_download (void)
{
  GError *error = NULL;
  gchar *out_obj_path = NULL;
  gchar *out_obj_path2 = NULL;
  NewportDownload *download_proxy;
  gboolean result;

  newport_service_call_get_download_for_url_sync (proxy, APP_URL, &out_obj_path,
                                                  NULL, NULL);
  download_proxy =
    newport_download_proxy_new_for_bus_sync (G_BUS_TYPE_SESSION,
                                             G_BUS_NAME_WATCHER_FLAGS_NONE,
                                             "org.apertis.Newport",
                                             out_obj_path, NULL, NULL);
  g_free (out_obj_path);
  result = newport_download_call_cancel_sync (download_proxy, NULL, &error);
  g_assert_no_error (error);
  g_assert (result);
  newport_service_call_get_download_for_url_sync (proxy, APP_URL,
                                                  &out_obj_path2, NULL, NULL);
  g_assert(out_obj_path2 == NULL);
}

static void
test_newport_resume_download (void)
{
  GError *error = NULL;
  gboolean state_result = FALSE;
  gchar *out_obj_path = NULL;
  NewportDownload *download_proxy;
  gulong dwn_state_id;
  gboolean result;
  gboolean timed_out = FALSE;
  guint timeout_id;

  newport_service_call_get_download_for_url_sync (proxy, APP_URL, &out_obj_path,
                                                  NULL, &error);
  download_proxy =
    newport_download_proxy_new_for_bus_sync (G_BUS_TYPE_SESSION,
                                             G_BUS_NAME_WATCHER_FLAGS_NONE,
                                             "org.apertis.Newport",
                                             out_obj_path, NULL, NULL);
  g_free (out_obj_path);

  dwn_state_id = g_signal_connect (download_proxy,
                                   "notify::state",
                                   G_CALLBACK (_download_state_changed),
                                   &state_result);

  result = newport_download_call_resume_sync (download_proxy, NULL, &error);
  g_assert_no_error (error);
  g_assert (result);
  timeout_id = g_timeout_add_seconds (100, _default_event_source, &timed_out);
  while (!timed_out && !state_result)
    g_main_context_iteration (NULL, TRUE);
  g_assert (!timed_out);
  g_source_remove (timeout_id);
  g_assert (state_result);
  g_signal_handler_disconnect (download_proxy, dwn_state_id);
}

static void
test_newport_start_download (void)
{

  GError *error = NULL;
  gboolean state_result = FALSE;
  gchar *arg_download_path;
  gchar *out_obj_path = NULL;
  gboolean result;
  GError *error1 = NULL;
  NewportDownload *download_proxy;
  gboolean timed_out = FALSE;
  gulong dwn_state_id;
  gulong dwn_progress_id;
  guint timeout_id;

  arg_download_path =
    g_build_filename (g_get_user_special_dir (G_USER_DIRECTORY_DOWNLOAD),
                      "newport-client", "zipfile", NULL);
  /* Call  download start to start downloading the file and wait till download-information signal quits the mainloop */
  result = newport_service_call_start_download_sync (proxy,
                                                     APP_URL,
                                                     arg_download_path,
                                                     &out_obj_path,
                                                     NULL, &error);
  g_assert_no_error (error);
  g_assert (result);
  g_free (arg_download_path);

  download_proxy =
    newport_download_proxy_new_for_bus_sync (G_BUS_TYPE_SESSION,
                                             G_BUS_NAME_WATCHER_FLAGS_NONE,
                                             "org.apertis.Newport",
                                             out_obj_path, NULL, &error1);
  g_free (out_obj_path);
  g_assert_no_error (error1);

  /*state_result flag set in download-information signal */
  g_signal_connect (download_proxy, "notify::error",
                    G_CALLBACK (_download_error_changed), &state_result);

  dwn_state_id = g_signal_connect (download_proxy, "notify::state",
                                   G_CALLBACK (_download_state_changed),
                                   &state_result);

  dwn_progress_id = g_signal_connect (download_proxy, "download-progress",
                                      G_CALLBACK (_download_info),
                                      &state_result);

  timeout_id = g_timeout_add_seconds (100, _default_event_source, &timed_out);
  while (!timed_out && !state_result)
    g_main_context_iteration (NULL, TRUE);
  g_assert (!timed_out);
  g_source_remove (timeout_id);
  g_assert (state_result);
  g_signal_handler_disconnect (download_proxy, dwn_progress_id);
  g_signal_handler_disconnect (download_proxy, dwn_state_id);

}

gint
main (gint argc, gchar *argv[])
{
  GTestDBus *test_dbus;
  GError *error = NULL;
  int ret;

  g_test_init (&argc, &argv, NULL);
  test_dbus = g_test_dbus_new (G_TEST_DBUS_NONE);
  g_test_dbus_add_service_dir(test_dbus,SERVICEFILEPATH);
  g_test_dbus_up(test_dbus);
  proxy = newport_service_proxy_new_for_bus_sync (
      G_BUS_TYPE_SESSION, G_BUS_NAME_WATCHER_FLAGS_NONE, "org.apertis.Newport",
      "/org/apertis/Newport/Service", NULL, &error);
  g_test_add_func ("/newpot-tests/test_newport_start_download",
                   test_newport_start_download);
  g_test_add_func ("/newpot-tests/test_newport_pause_download",
                   test_newport_pause_download);
  g_test_add_func ("/newpot-tests/test_newport_resume_download",
                   test_newport_resume_download);
  g_test_add_func ("/newpot-tests/test_newport_cancel_download",
                   test_newport_cancel_download);
  g_test_add_func ("/newpot-tests/test_newport_complete_download",
                   test_newport_complete_download);
  g_test_add_func ("/newpot-tests/test_newport_get_download_list",
                   test_newport_get_download_list);
  ret = g_test_run ();
  g_test_dbus_down(test_dbus);
  return ret;
}

